function MainDDS

% this is the main Dynamically Dimensioned Search (DDS) algorithm *calling* program for running the DDS algorithm
% multiple times on a single problem.  Originally developed so that compiled version could be sent to the 
% Cornell Theory Center.  Now it functions as general code for distributing executable program to all DDS users.
% Program is for calling and accumulating MULTIPLE optimization trial results of the DDS algorithm.  
% This code can be succesfully executed from the Matlab command prompt AND orignially could be compiled and then
% executed from DOS prompt.
% Code is easily modified (~30 minutes) for another algorithm besides DDS - see USER NOTES below.

% Written by Bryan Tolson - Jan 2003 to Dec 2004 ... small updates up to
% Feb 2015

% General program for either MAX or MIN problem and any 
%   A) matlab function coded as an m file OR
%   B) compiled model exe (i.e. a simulation model like SWAT) 
%   In case A), the .m function must accept as input argument the decision variable values and return
%   as a single output the value of the objective function.  Similar idea
%   in case B) except that the DDS algorithm calls a Matlab .m file that
%   must communicate with the compiled exe file you are trying to run.

% Program requires TWO input files from the USER:
%   1.  Algorithm input control file called 'DDS_inp.txt'
%       See this file and comments below for input descriptions.
%   2.  Objective or cost function decision variable description file
%       called 'strcat(objfunc_name,'.inp')' where the objfunc_name variable
%       is the name of the function that returns the objective function
%       value to the DDS optimization algorithm.  So, in the example files
%       provided,  we minimize the 'ackley10.m' function and thus need an
%       input file describing the variable bounds called 'ackley10.inp'.
%       1 headerline, then column 1 is decision variable names, column 2 is
%       the lower bounds, and column 3 is the upper bounds.
%       See format of ackley10.inp - as long as 1 headerline and spaces between column,
%       code reads file fine.

% Saves trial outputs to multiple files (see code in DDS_inout.m for better description of output)
% - user picks amount of data (#files) saved.  Algorithm saves Jbest, Jtest
% for all iterations of all optimization trials as well as best solutions
% per trial and the initial solutions as well as a summary of algorithm
% inputs so that the results can be replicated exactly.

% FUNCTIONS CALLED by this program:
    % DDS.m  - main DDS algorithm code for one trial.  
    % bounds.m - reads in decision variable info from the 'strcat(objfunc_name,'.inp')' file.
    % read_DDS_inp.m - reads the DDS algorithm inputs from the DDS_inp.txt file.
    % DDS_inout.m - echos USER inputs to an output file.
    % plot_v1.m - postporcessing code to roughly plot up all aglorithm results

% All other .m files in this directory are called by one or more of the
% above 5 functions and include textread_bt.m, neigh_value.m, cleanInput.m,
% ext_function.m, get_objfunc.m.
% The ackley10.m function is a problem specific example obj function to minimize. 

% disp('This program saves all objective funtion (J) info such as Jtest,')
% disp('Jbest, as well as the')
% disp('decision variable values for all iterations and all optimization trials.')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% IMPORTANT USER NOTES %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 1.  OPTIMIZATION ALGORITHM NOTES.  
%     Modifying this code for your algorithm is very easy if your algorithm is
%     NOT population based (If it is, it is still possible, just need to
%     be careful).  In either case, you need to change the function code in the locations
%     marked by '***NEW ALGORITHM***' so that this code calls your
%     algorithm with the correct inputs, and your algorithm outputs are
%     converted to the variables expected by this function.  
% 2.  OBJECTIVE OR COST FUNCTION.  Code is general below for any function to 
%     be optimized (i.e. no code changes needed for new cost function) but it can 
%     easily be tweaked if necessary for running a complicated simulation model during 
%     optimization (i.e. calibration or management optimization). See lines commented 
%     with '***AUTOCAL***' for where users may want to modify this function.  
%     Another tweak may be to increase precision of output [fprintf(fid,...) ] writing.
%     A final tweak may be to make the current *algorithm* iteration number
%     in each trial (from your algorithm function) a global variable here
%     as well as in your cost function so that your cost function can
%     contain problem specific coding that writes extra simulation model output to 
%     user defined output files and can associate them with algorithm iteration number.  
%     Note that if these are called *.out files, the code below will automatically MOVE (not copy) 
%     them to your specified output directory (same directory containing optimization outputs).
%     The trial_num variable below is declared globally so it is also
%     available for the same purposes of current iteration number.  
%     i.e. you could have code in your cost function to parse through
%     simulation model outputs so you can save 1 output time series, say daily flow at
%     one location, to a file called dailyflow2_23.out where #2 is for trial
%     number 2 and 23 is for algorithm iteration #23.
%     Note that trial_num=j in the algorithm calling loop below --> do a similar thing with the 
%     current iteration number (like iteration_num=i) in your algorithm iteration loop
%     & make iteration_num the global variable.
%     Above discussion may be helpful for people using this code to
%     implement DDS-AU method (Tolson and Shoemaker, 2008)
% 3.  RANDOM NUMBER GENERATION.
%     Note that the random number generation code has been modified to
%     reflect the new MATLAB syntax
%     the 'rng' function controls the shared generator used by rand, randn, 
%     randi and all other random number generation functions 
%     It does not require the user to specify the Generator input
%     The default Mersenne Twister generator is used at
%     MATLAB startup. In future releases when new generators supersede the 
%     Mersenne Twister, this code will use the new default. (Line 270-275)


clear;
fclose('all');

% DEFINE A FEW GLOBAL VARIABLES FOR USE BY algorithm and other functions:
global objfunc_name; % function name that returns the objective function value

global S_name; % decision variable names
global S_file; %  Cannon Autocalibration problem Specific
global S_linenum;  % Cannon Autocalibration problem Specific
global S_min;  % decision variable minimums
global S_max;  % decision variable maximums
global Discrete_flag;  % decision variable type: continuous (0) or integer (1)

global Modeldir; % directory containing user specified objective function evaluation code
global codedir; % current directory containing this code
global trial_initial % finalized initial solution used in a single DDS opt trial

global trial_num; % ***AUTOCAL*** number of current trial running so your cost function can  
                  % know this for for saving supplemental model run outputs in unique files.
% global iteration_num % ***AUTOCAL*** described above in USER NOTE #2
global exe_name;
                           
codedir=pwd;  % get the current directory for output file management

a=[]; % temp storage of initial solution
A=[]; % matrix of initial solutions for all trials 

%%% Screen message:
fprintf('\n\n');
fprintf('------------- DDS Optimization Algorithm Main Program ------------- \n\n');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Gather USER PROGRAM inputs:
[objfunc_name, runname, obj_flag, num_trials, maxiter, user_seeds, fraction1, ...
        out_print, user_comments, ini_soln, ini_name, dir_path] = read_DDS_inp([]);

% ***NEW ALGORITHM*** 1.  Edit and rename read_DDS_inp.m function to read algorithm run inputs.
%                     2.  Modify OR delete the returned variables named ALGORITHM SPECIFIC below
% objfunc_name - name of matlab .m file (no .m) OR an external exe function (with .exe) 
%               OR an external batch file (*.bat) that returns objective function.  GENERAL.
% runname - short name used to create output files.  GENERAL. 
% obj_flag - flag for objective: "-1" if maximization, "1" if minimization --> when
%       algorithm called it is a minimizer.  ALGORITHM SPECIFIC.
% num_trials - number of optimization trials to do.  GENERAL.
% maxiter - is the maximum number of iterations or function calls in each optimization
%       trial.  Always runs to maximum iterations for DDS.  GENERAL.
% user_seeds - 2 element column vector of integer random number seeds to fix random #
%               generator state.  GENERAL. #'s between 1 and 10^9
% fraction1 - std dev/DV range to determine variance of normal
%           perturbation.  ALGORITHM SPECIFIC.
% out_print - flag for minimizing number of output files. 
%           "0" to save all information or "1" to save only summary info.
%           GENERAL.
% user_comments - self explanatory.  GENERAL.
% ini_soln - flag for if initial solutions are provided (1), or random
%           initial solutions to be used (0).  GENERAL.
% ini_name - if ini_soln is 1, then this is a string for filename containing 
%           initial solutions for all trials, else it is blank.  GENERAL.
% dir_path - name of subdirectory in this directory (or enter a full directory path) 
%           where a simulation model is executed (optional).  GENERAL.

if isempty(findstr(objfunc_name,'.exe'))==0 % then calling an exe file
    exe_name=objfunc_name; 
    objfunc_name='ext_function'; % hardcoded, general external function interface file
elseif isempty(findstr(objfunc_name,'.bat'))==0 % then calling a bat file
    exe_name=objfunc_name; 
    objfunc_name='ext_function'; % hardcoded, general external function interface file
else
    exe_name=[]; % running in Matlab, no exe or bat file used for obj function
end

if isempty(dir_path)
    Modeldir=codedir;
elseif isempty(strfind(dir_path,':\'))  % ':\' not found --> a subdirectory entered
    Modeldir=strcat(codedir,'\',dir_path);
else  % new option, ':\' found:
    Modeldir=dir_path;  % absolute path entered in input file
end    
% note: no need to record name of model subdirectory

% boundname='SWAT_par.inp'; % Cannon Autocal Specific %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% boundname=strcat(objfunc_name,'.inp');
% fprintf('Program expects %s file with decision variable bounds to be in current directory.\n',boundname)
% fprintf('bounds.m reads file called %s. \n\n',boundname);

% call the bounds program to return decision variable bounds:
bounds;  
S_name=char(S_name);
[num_dec , ~]=size(S_min);

% Error check bounds file:
if Discrete_flag == 1
   % Need to verify that bounds entered for discrete decision variables are
   % not continuous
   test_min = rem(S_min,1); 
   test_max = rem(S_max,1); 
   if any(test_min(:)) == 1 || any(test_max(:)) == 1
       error('You have entered continuous bounds for a discrete decision variable! Please input discrete bounds and try again.')
   end
end
    

% ALGORITHM AND OUTPUT OPERATE with S = [ # # # #] -> row vectors of decision variables:
    S_min=S_min';
    S_max=S_max';
% remember these are globally available for your algorithm to use

% Initial solution coding:
%   (generate random initial solutions in DDS)
    Initial_mat=[];
    if ini_soln==0 % user didn't define initial solution
        num_samples=max(5,round(0.005*maxiter)); % try 5 or more MCS samples *******
    elseif ini_soln==1 % user has defined starting solutions already, get all initials
        fid=fopen(ini_name,'r');
        Initial_mat=fscanf(fid,'%e',[num_dec inf]);
        Initial_mat=Initial_mat';
        fclose(fid);
         num_samples=1; % assume 1st solution "counts"
         [num_trials2 num_dec2] = size(Initial_mat);
         if num_trials2~=num_trials
             error('Number of initial solutions does not match # trials selected.')
         end
         if num_dec2~=num_dec
             error('# of dec vars in S_min & initial solution matrix not consistent.')
         end
     end

%define some output file names:
filenam1=strcat(runname,'_ini.out'); % initial sol'n file name
filenam2=strcat(runname,'_AVG.out'); % avg trial outputs
filenam3=strcat(runname,'_sbest.out'); % output best DV solutions per trial
filenam4=strcat(runname,'_trials.out'); % output Jbest per iteration number per trial

% Echo USER inputs to output file: ************************************************************************
DDS_inout(runname,obj_flag,num_trials,maxiter,user_seeds,fraction1,out_print,user_comments, ...
    ini_soln,ini_name,num_samples,filenam1,filenam2,filenam3,filenam4); 

%  ***NEW ALGORITHM***  1.  Create a new function based on DDS_inout that
%                       records optimization run inputs for your algorithm.
%                       2.  Change input arguments for your algorithm specific parameters.
% num_samples - number of random samples used to initialize each opt trial
%               from random initial solutions.  Something more than 1 works
%               best!   GENERAL unless you want to force this to be 1
%               always.
% filname*  - names of output files, see code comments above after filenam# variables assigned
% ALL other DDS_inout input arguments defined earlier.

% Input error checking:
if obj_flag~=-1 & obj_flag~=1  % ***NEW ALGORITHM*** only if your algorithm uses variable obj_flag, else comment if
    error('Please enter -1 or 1 for objective function flag!  Try program again.')
end
if num_trials<1 | num_trials>1000
    error('Please enter 1 to 1000 optimization trials!  Try program again.')
end
if maxiter<7 | maxiter>1000000
    error('Please enter 7 to 1000000 for max # function evaluations!  Try program again.')
end
if out_print~=0 & out_print~=1
    error('Please enter 0 or 1 for output printing flag! Try program again.')
end
% done input error checking

% intialize some variables
Jbest_final=[]; % Jbest after max iterations
Sbest=[]; % matrix holding the best sets of decision variables
it_sbest=[]; % iteration number the local/global optimum is found at for each trial
% for variables below, see their definition integrated in code where they are used
output=[];
outputALL=[];
time_trial=0;
all_sum=[];
all_sum=zeros(maxiter,3);  % output matrix size fixed.  Initial solution evaluation is iteration# 0!
Jbest_trials=[]; % tracks only Jbest but for all trials in one file
Jbest_trials=zeros(maxiter,num_trials);
Jbest_avg=[];
Jbest_avg=zeros(maxiter,1); % Initial solution evaluation is iteration# 0!

%%% ***AUTOCAL*** Specific (next 3 lines) %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% call supplemental functions to get meas data and initialize variables for
%%% efficient model/data stat comparisons:
%get_measdata; % 
% add code above if it is helpful for your simulation model 

% Fix Random number generator:
% Note: Reproduce same random numbers each time
% using updated MATLAB syntax - Mersenne Twister
% ---------------------------------------------------
uni_seed=round(mean(user_seeds));
rng(uni_seed);


for j=1:num_trials % ######   ALGORITHM TRIAL CALLING LOOP   #####################################
    trial_num=j; % %%% Can-Autocal Specific, *** AUTOCAL *** just note this is here
    %screen output:
     fprintf('Trial number %g executing ... \n',j);
    
    % Initialize and pre-allocate relevant variables each trial
    output=[];
    output=zeros(maxiter,3); % iter#, Jcurrent, Jbest
    outputALL=[];
    outputALL=zeros(maxiter,3+num_dec); % Columns: iter#, Jbest, Jtest, dec var values 
                                    % Jtest is the neighbor sampled at each iteration 
        tic;  % time the trial
        
        % set the initial solution (vector or blank)
        if isempty(Initial_mat)
            Initial=[];
        else
            Initial=Initial_mat(j,:);
        end
        
        %%%%%%%%%%% Call the DDS optimization algorithm  ***NEW ALGORITHM***
        [outputALL, it_sbest, sbest, trial_initial]=DDS(Initial,num_samples,obj_flag,fraction1,maxiter); 
        output=outputALL(:,1:3);
        % outputALL is maxiter+1 rows (initial solution is iteration 0) and columns are:    
            % column 1 =iter#, column 2 =best f(x) or Jbest, column 3 = test f(x) or Jtest,
            % column 4 = decision variable 1 (dec var 1) value for test f(x),
            % column 5 = dec var 2 value for test f(x), ...
            % column (3+num_dec) = dec var (3+num_dec) value for Jtest (i.e. last dec variable value)
        % it_sbest - is iter# when best solution is found
        % sbest  - row vector of best solution decision variable values
        %  ***NEW ALGORITHM*** 1.  User needs to call new optimization
        %                          algorithm with some new algorithm parameters.
        %                      2.  Optimization algorithm output must be converted by user to
        %                          assign expected values to outputALL, it_sbest, and sbest variables.
        %                          Note that it_sbest is not critical.
        %                          Conversion can be here, right after algorithm call, OR can occur 
        %                          in your optimization algorithm if you wish.  For an algorithm like 
        %                          SA, you may also wish to track output like 'Jcurrent' or 
        %                          #uphill moves accepted - in this case add code accordingly here.
        
        % calc trial exe time:
        time_trial=toc;
        time_trialhr=time_trial/(3600);

        % get initial DDS solution from DDS output:
        A(j,:)=trial_initial;
                
        % accumulate summary algorithm results for all trials (sum them):
        all_sum=all_sum+output; % col 1 is sum of iteration numbers
                                % col 2 is sum of Jtest values
                                % col 3 is sum of Jbest values                              
        Sbest=[Sbest; sbest];
        
        % accumlate only Jbest for each trial:
        Jbest_trials(:,j)=output(:,2);
        
     if out_print==0 % write file for trial outputs: (else only prints summary output)
        % write file for trial outputs:
        jchar=int2str(j);
        filenam=strcat(runname,'_',jchar,'.out');
        fid = fopen(filenam,'w'); % opens file and discards current contents
        fprintf(fid,'Time of execution for trial %g was %g seconds or %g hours. \n\n',j,time_trial,time_trialhr);
        fprintf(fid,'Iter#      Jbest         Jtest          Decision Variables for Jtest: 1, 2, 3 ... ->  \n');
        
        for k=1:maxiter % write output lines:
              fprintf(fid,'%6.0f  %12.5f %12.5f',output(k,:));
              fprintf(fid,'     %10.5f',outputALL(k,4:3+num_dec));
              fprintf(fid,'\n');
        end
        fclose(fid);
        
        filenam=strcat(runname,'_sbest',jchar,'.out');
        fid_sbest=fopen(filenam,'w');
        fprintf(fid_sbest,' %e ',sbest); 
        fclose(fid_sbest);
     else % rewrite same trial output files after each iteration so intermediate outputs saved
        filenam=strcat(runname,'_sbest.out');
        fid_Sbest=fopen(filenam,'w');
        for i=1:j % # trials completed
            fprintf(fid_Sbest,' %e ',Sbest(i,:)); 
            fprintf(fid_Sbest,'\n');
         end
         fclose(fid_Sbest);
         
        fid_Jbest=fopen(filenam4,'w');
        for i=1:maxiter
            fprintf(fid_Jbest,' %e ',Jbest_trials(i,:)); 
            fprintf(fid_Jbest,'\n');
        end
        fclose(fid_Jbest);
     end

     %screen output:
     fprintf('     Best objective function value is %f \n',output(maxiter,2));
end % ######   END ALGORITHM TRIAL CALLING LOOP   #####################################

fprintf('\n');

%%%%%%%%%%%%%%%%%%%%%%%%%% SAVE, CALCULATE AND SAVE OUTPUT FOR ALGORITHM:

% write initial solutions to output file:
     fid_ini=fopen(filenam1,'w');
     for i=1:num_trials
        fprintf(fid_ini,' %e ',A(i,:)); 
        fprintf(fid_ini,'\n');
     end
     fclose(fid_ini);

if out_print==0  % write files (else they are already written in loop above)
% save only JBest for all trials in one file: 
     fid_Jbest=fopen(filenam4,'w');
     for i=1:maxiter
        fprintf(fid_Jbest,' %e ',Jbest_trials(i,:)); 
        fprintf(fid_Jbest,'\n');
     end
     fclose(fid_Jbest);

        filenam=strcat(runname,'_sbest.out');
        fid_Sbest=fopen(filenam,'w');
        for i=1:num_trials
            fprintf(fid_Sbest,' %e ',Sbest(i,:)); 
            fprintf(fid_Sbest,'\n');
         end
         fclose(fid_Sbest);
end

% compute averages:  
    % NOT CORRECT IF ALGORITHM TERMINATES EARLY (CONVERGES AND STOPS)
    % OK THOUGH FOR DDS
Jtest_avg=(all_sum(:,3))/num_trials;
Jbest_avg=(all_sum(:,2))/num_trials;
%  ***NEW ALGORITHM***  1.  IF your algorithm can terminate before maximum iteration limit
%                           then you must change above avg calcs or not use since they will not work. 

% write summary output to the screen:
trial_number=(1:num_trials)';

iter_number=[1:maxiter]';

% write summary (avgs) output to a file:
        filenam=strcat(runname,'_AVG.out');
        fid = fopen(filenam,'w'); % opens file and discards current contents
        fprintf(fid,'Iter#        Jbest_Avg       Jtest_Avg \n');
        tempv2=[iter_number'; Jbest_avg'; Jtest_avg'];
        fprintf(fid,'%6.0f     %12.5f      %12.5f       \n',tempv2);     
        fclose(fid);

fclose all;


%%%%%%%%%%%%%%%%%%% OUTPUT FILE MANAGEMENT:
% all the code below works in both the compiled and command prompt run
% modes (triple checked)

% move all OPTIMIZATION output files in codedir to a new directory called shortname:
% assume (we must be!!) we are currently in this directory.

%***AUTOCAL***  READ BELOW:      
% IF YOU are saving  simulation model output files (i.e. output time series per iteration or something), 
% and IF obj function coding is saving save these as '*.out' files in the codedir, 
% THEN, the code below will move all these .out files (along with
% optimization output files) to the runname directory.

command1=[];
command1=['mkdir ' runname]; % works in Matlab prompt and compiled
dos(command1);
%     % do optimization output first: (general)
      tempname=strcat('*.out');  %  this statement saves all *.out files
      opt_out_files=dir(tempname);
      for i=1:length(opt_out_files)
          command1=[];
          command1=['move ' opt_out_files(i).name ' ' runname];  % works in Matlab prompt and compiled
          dos(command1);
      end
  
 file=strcat(objfunc_name,'.inp');
 command=['copy ' file ' ' runname];  
 [zz, xx]=dos(command);
 command=[];
 
 command=['copy DDS_inp.txt ' runname];  
 [zz, xx]=dos(command);

 % done output file mgt
 
  %%% Screen message:
 fprintf('MainDDS Program finished.  See output files in %s directory.\n\n',runname);

 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% POSTPROCESSING CODE:
response=input('\n    Please enter "yes" to plot & analyze outputs: ','s');
if (strcmp(response,'yes')| strcmp(response,'Yes'))
    plot_v1(objfunc_name);
end

 % users may want to add some function calls here to do output common processing so it is done 
 % automatically here...
 